#! /usr/bin/python
#   Author:  Arjun Sarwal   arjun@laptop.org
#    Copyright (C) 2007, OLPC
#    
#    	
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import pygst
pygst.require("0.10")
import gtk, gst
import gobject
import signal, os
import time
import dbus
import audioop
import subprocess
from struct import *
from string import *
import gtk

class AudioGrab(gobject.GObject):

    def __init__(self, callable1):
        gobject.GObject.__init__(self)
        
        rate=4000                                              #Default sampling rate at startup of device
        self.handoff_state = True
        self.callable1 = callable1
        
        self.pipeline = gst.Pipeline("pipeline")
        self.alsasrc = gst.element_factory_make("alsasrc", "alsa-source")
        self.pipeline.add(self.alsasrc)
        self.caps1 = gst.element_factory_make("capsfilter", "caps1")
        self.pipeline.add(self.caps1)
        caps_str = "audio/x-raw-int,rate=%d,channels=1,depth=16" % (rate, )
        self.caps1.set_property("caps", gst.caps_from_string(caps_str) )
        self.fakesink = gst.element_factory_make("fakesink", "fsink")
        self.pipeline.add(self.fakesink)		
        self.fakesink.connect("handoff", self.on_buffer)	
        self.fakesink.set_property("signal-handoffs",True) 
        gst.element_link_many(self.alsasrc, self.caps1, self.fakesink)

        self.buffer_taken = False

    def __del__(self):
        self.set_dc_mode(False)
        self.set_bias(False)
        self.set_capture_gain(70)
        self.set_mic_boost(False)
        #print "destructor function called"

    def set_handoff_signal(self, handoff_state):
        """Sets whether the handoff signal would generate an interrupt or not"""
        self.fakesink.set_property("signal-handoffs",handoff_state)
        #print "handoff ", handoff_state

    def _new_buffer(self):
        """This function is NOT used"""
        self.fakesink.set_property("signal-handoffs", False)
        self.pipeline.set_state(gst.STATE_NULL)
        print "disabling audio device"		
        
    
    
    def on_buffer(self, element, buffer, pad):		
        """The function that is called whenever new data is available
        This is the signal handler for the handoff signal"""
        if self.buffer_taken == False:
            gobject.idle_add(self.callable1, buffer)
            self.buffer_taken = True
        else:
            pass
            #self.stop_sound_device()
            #print "passing in audiograb"	
            #gobject.idle_add(self._new_buffer)
        return False
    

    def start_sound_device(self):
        """Start or Restart grabbing data from the audio capture"""
        #print "actual start of sound device from within audiograb"	
        gst.event_new_flush_start()
        self.pipeline.set_state(gst.STATE_PLAYING)


    def stop_sound_device(self):
        """Stop grabbing data from capture device"""
        #print "stopping sound device"
        gst.event_new_flush_stop()
        self.pipeline.set_state(gst.STATE_NULL)
        
    
    def mute_master(self):
        """Mutes the Master Control"""
        os.system("amixer set Master mute")
    
    def unmute_master(self):
        """Unmutes the Master Control"""
        os.system("amixer set Master unmute")
    
    def mute_PCM(self):
        """Mutes the PCM Control"""
        os.system("amixer set PCM mute")
    
    def unmute_PCM(self):
        """Unmutes the PCM Control"""
        os.system("amixer set PCM unmute")
    
    def mute_mic(self):
        """Mutes the Mic Control"""
        os.system("amixer set Mic mute")
    
    def unmute_mic(self):
        """Unmutes the Mic Control"""
        os.system("amixer set Mic unmute")
    
    def set_master(self, master_val ):
        """Sets the Master gain slider settings 
        master_val must be given as an integer between 0 and 100 indicating the
        percentage of the slider to be set"""
        os.system("amixer set Master " + str(master_val) + "%")
        
        
    def get_master(self):
        """Gets the Master gain slider settings. The value returned is an integer between 0-100
        and is an indicative of the percentage 0 - 100%"""
        p = str(subprocess.Popen(["amixer", "get", "Master"], stdout=subprocess.PIPE).communicate()[0])
        p = p[find(p,"Front Left:"):]
        p = p[find(p,"[")+1:]
        p = p[:find(p,"%]")]
        return int(p)
    
    
    
    def get_mix_for_recording(self):
        """Returns True if Mix is set as recording device and False if it isn't """
        p = str(subprocess.Popen(["amixer", "get", "Mix", "capture", "cap"], stdout=subprocess.PIPE).communicate()[0])
        p = p[find(p,"Mono:"):]
        p = p[find(p,"[")+1:]
        p = p[:find(p,"]")]
        if p=="on" :
            return True
        else:
            return False
            

    def get_mic_for_recording(self):
        """Returns True if mic is set as recording device and False if it isn't """
        p = str(subprocess.Popen(["amixer", "get", "Mic", "capture", "cap"], stdout=subprocess.PIPE).communicate()[0])
        p = p[find(p,"Mono:"):]
        p = p[find(p,"[")+1:]
        p = p[:find(p,"]")]
        if p=="on" :
            return True
        else:
            return False
    
    def set_mic_for_recording(self):
        """Sets Mic as the default recording source"""
        os.system("amixer set Mic capture cap")
    
    def set_mix_for_recording(self):
        """Sets Mix as the default recording source"""
        os.system("amixer set Mix capture cap")
        
    
    def set_bias(self,bias_state=False):
        """Sets the Bias control
        pass False to disable and True to enable"""
        if bias_state==False:
            bias_str="mute"
        else:
            bias_str="unmute"
        os.system("amixer set 'V_REFOUT Enable' " + bias_str)
   
    def get_bias(self):
        """Returns the setting of Bias control 
        i.e. True: Enabled and False: Disabled"""
        p = str(subprocess.Popen(["amixer", "get", "'V_REFOUT Enable'"], stdout=subprocess.PIPE).communicate()[0])
        p = p[find(p,"Mono:"):]
        p = p[find(p,"[")+1:]
        p = p[:find(p,"]")]
        if p=="on" :
            return True
        else:
            return False
   
    def set_dc_mode(self, dc_mode = False):
        """Sets the DC Mode Enable control
        pass False to mute and True to unmute"""
        if dc_mode==False:
            dcm_str="mute"
        else:
            dcm_str="unmute"
        os.system("amixer set 'DC Mode Enable' " + dcm_str)

    def get_dc_mode(self):
        """Returns the setting of DC Mode Enable control 
        i .e. True: Unmuted and False: Muted"""
        p = str(subprocess.Popen(["amixer", "get", "'DC Mode Enable'"], stdout=subprocess.PIPE).communicate()[0])
        p = p[find(p,"Mono:"):]
        p = p[find(p,"[")+1:]
        p = p[:find(p,"]")]
        if p=="on" :
            return True
        else:
            return False
        
    def set_mic_boost(self, mic_boost=False):
        """Sets the Mic Boost +20dB control
        pass False to mute and True to unmute"""
        if mic_boost==False:
            mb_str="mute"
        else:
            mb_str="unmute"
        os.system("amixer set 'Mic Boost (+20dB)' " + mb_str)
        
    def get_mic_boost(self):
        """Returns the setting of Mic Boost +20dB control 
        i.e. True: Unmuted and False: Muted"""
        p = str(subprocess.Popen(["amixer", "get", "'Mic Boost (+20dB)'"], stdout=subprocess.PIPE).communicate()[0])
        p = p[find(p,"Mono:"):]
        p = p[find(p,"[")+1:]
        p = p[:find(p,"]")]
        if p=="on" :
            return True
        else:
            return False
            
    
    def set_capture_gain(self, capture_val):
        """Sets the Capture gain slider settings 
        capture_val must be given as an integer between 0 and 100 indicating the
        percentage of the slider to be set"""
        os.system("amixer set Capture " + str(capture_val) + "%")
        
        
    def get_capture_gain(self):
        """Gets the Capture gain slider settings. The value returned is an integer between 0-100
        and is an indicative of the percentage 0 - 100%"""
        p = str(subprocess.Popen(["amixer", "get", "Capture"], stdout=subprocess.PIPE).communicate()[0])
        p = p[find(p,"Front Left:"):]
        p = p[find(p,"[")+1:]
        p = p[:find(p,"%]")]
        return int(p)
        
    
    def set_PCM_gain(self, PCM_val):
        """Sets the PCM gain slider settings 
        PCM_val must be given as an integer between 0 and 100 indicating the
        percentage of the slider to be set"""
        os.system("amixer set PCM " + str(PCM_val) + "%")
        
    def get_PCM_gain(self):
        """Gets the PCM gain slider settings. The value returned is an indicative of the percentage 0 - 100%"""
        p = str(subprocess.Popen(["amixer", "get", "PCM"], stdout=subprocess.PIPE).communicate()[0])
        p = p[find(p,"Front Left:"):]
        p = p[find(p,"[")+1:]
        p = p[:find(p,"%]")]
        return int(p)
        
        
    def set_mic_gain(self, mic_val):
        """Sets the MIC gain slider settings 
        mic_val must be given as an integer between 0 and 100 indicating the
        percentage of the slider to be set"""
        os.system("amixer set Mic " + str(mic_val) + "%")
        
    def get_mic_gain(self):
        """Gets the MIC gain slider settings. The value returned is an indicative of the percentage 0 - 100%"""
        p = str(subprocess.Popen(["amixer", "get", "Mic"], stdout=subprocess.PIPE).communicate()[0])
        p = p[find(p,"Mono:"):]
        p = p[find(p,"[")+1:]
        p = p[:find(p,"%]")]
        return int(p)
    
    def set_sampling_rate(self, sr):
        """Sets the sampling rate of the capture device
        Sampling rate must be given as an integer for example 16000 for setting 16Khz sampling rate
        The sampling rate would be set in the device to the nearest available"""
        caps_str = "audio/x-raw-int,rate=%d,channels=1,depth=16" % (sr, )
        self.caps1.set_property("caps", gst.caps_from_string(caps_str) )


    def get_sampling_rate(self):
        """Gets the sampling rate of the capture device"""
        return int(self.caps1.get_property("caps")[0]['rate'] )

    def set_callable1(self, callable1):
        """Sets the callable to the drawing function for giving the
        data at the end of idle-add"""
        self.callable1 = callable1
        
    
    def save_settings(self):
        """Save audio device settings at startup"""
    
    def save_settings_temp(self):
        """Save settings temporarily. For example if activity is going to be inactive for some time"""
    
    def apply_settings_temp(self):
        """Re-apply settings that were saved temporarily. For example Activity becomes active again"""
    
    def apply_settings(self):
        """Apply settings that were saved at the beginning. Ideally should be called on quitting, and after pipeline has been stopped"""

    def set_sensor_type(self, sensor_type=1):
        """Set the type of sensor you want to use. Set sensor_type according to the following
        0 - AC coupling with Bias Off --> Very rarely used. Use when connecting a dynamic microphone externally
        1 - AC coupling with Bias On --> The default settings. The internal MIC uses these
        2 - DC coupling with Bias Off --> Used when using a voltage output sensor. For example LM35 which gives output proportional to temperature
        3 - DC coupling with Bias On --> Used with resistive sensors. For example"""
        if sensor_type==0:
            self.set_dc_mode(False)
            self.set_bias(True)
            self.set_capture_gain(70)
            self.set_mic_boost(True)
        elif sensor_type==1:
            self.set_dc_mode(False)
            self.set_bias(True)
            self.set_capture_gain(40)
            self.set_mic_boost(True)
        elif sensor_type==2:
            self.set_dc_mode(True)
            self.set_bias(False)
            self.set_capture_gain(0)
            self.set_mic_boost(False)
        elif sensor_type==3:
            self.set_dc_mode(True)
            self.set_bias(True)
            self.set_capture_gain(0)
            self.set_mic_boost(False)
        
